# Generated file
# - do not overwrite
# - do not include in git
from collections import namedtuple

BuildInfo = namedtuple(
    'BuildInfo',
    ['version', 'built_at', 'sha', 'branch', 'tag'],
)

BUILD_INFO = BuildInfo(
    version='v4.10.5~64d4535',
    built_at='2024-09-13 10:11:53Z',
    sha='64d4535660de5e3668f42289574c581a8959364a',
    branch='HEAD',
    tag='v4.10.5',
)



from typing import Optional

from .logging import prepare_logging, LOG_FILTER


def _config_to_string(config: object):
    lines = [f'{type(config).__name__}']
    fields = (f for f in config.__dict__.keys() if not f.startswith('_'))
    for field in fields:
        v = str(getattr(config, field))
        t = type(getattr(config, field)).__name__
        lines.append(f'- {field} = {v} [{t}]')
    return '\n'.join(lines)


class ConfigModel:

    def __str__(self):
        return _config_to_string(self)


class GeneralConfig(ConfigModel):

    def __init__(self, environment: str, client_url: str, secret: str):
        self.environment = environment
        self.client_url = client_url
        self.secret = secret


class SentryConfig(ConfigModel):

    def __init__(self, enabled: bool, workers_dsn: Optional[str],
                 traces_sample_rate: Optional[float], max_breadcrumbs: Optional[int]):
        self.enabled = enabled
        self.workers_dsn = workers_dsn
        self.traces_sample_rate = traces_sample_rate
        self.max_breadcrumbs = max_breadcrumbs


class DatabaseConfig(ConfigModel):

    def __init__(self, connection_string: str, connection_timeout: int,
                 queue_timeout: int):
        self.connection_string = connection_string
        self.connection_timeout = connection_timeout
        self.queue_timeout = queue_timeout


class S3Config(ConfigModel):

    def __init__(self, url: str, username: str, password: str,
                 bucket: str, region: str):
        self.url = url
        self.username = username
        self.password = password
        self.bucket = bucket
        self.region = region


class LoggingConfig(ConfigModel):

    def __init__(self, level: str, global_level: str, message_format: str,
                 dict_config: Optional[dict] = None):
        self.level = level
        self.global_level = global_level
        self.message_format = message_format
        self.dict_config = dict_config

    def apply(self):
        prepare_logging(self)

    @staticmethod
    def set_logging_extra(key: str, value: str):
        LOG_FILTER.set_extra(key, value)


class AWSConfig(ConfigModel):

    def __init__(self, access_key_id: Optional[str], secret_access_key: Optional[str],
                 region: Optional[str]):
        self.access_key_id = access_key_id
        self.secret_access_key = secret_access_key
        self.region = region

    @property
    def has_credentials(self) -> bool:
        return self.access_key_id is not None and self.secret_access_key is not None


class CloudConfig(ConfigModel):

    def __init__(self, multi_tenant: bool):
        self.multi_tenant = multi_tenant



import os
import yaml

from typing import List, Any, IO

from .keys import ConfigKey, ConfigKeys
from .model import GeneralConfig, SentryConfig, S3Config, \
    DatabaseConfig, LoggingConfig, CloudConfig, AWSConfig


class MissingConfigurationError(Exception):

    def __init__(self, missing: List[str]):
        self.missing = missing


class DSWConfigParser:

    def __init__(self, keys=ConfigKeys):
        self.cfg = dict()
        self.keys = keys

    @staticmethod
    def can_read(content: str):
        try:
            yaml.load(content, Loader=yaml.FullLoader)
            return True
        except Exception:
            return False

    def read_file(self, fp: IO):
        self.cfg = yaml.load(fp, Loader=yaml.FullLoader) or self.cfg

    def read_string(self, content: str):
        self.cfg = yaml.load(content, Loader=yaml.FullLoader) or self.cfg

    def has_value_for_path(self, yaml_path: list[str]):
        x = self.cfg
        for p in yaml_path:
            if not hasattr(x, 'keys') or p not in x.keys():
                return False
            x = x[p]
        return True

    @staticmethod
    def _prefix_var(var_name: str) -> str:
        return f'DSW_{var_name}'

    def has_value_for_key(self, key: ConfigKey):
        if self.has_value_for_path(key.yaml_path):
            return True
        for var_name in key.var_names:
            if var_name in os.environ.keys() or \
                    self._prefix_var(var_name) in os.environ.keys():
                return True

    def get_or_default(self, key: ConfigKey):
        x = self.cfg  # type: Any
        for p in key.yaml_path:
            if not hasattr(x, 'keys') or p not in x.keys():
                return key.default
            x = x[p]
        return x

    def get(self, key: ConfigKey):
        for var_name in key.var_names:
            if var_name in os.environ.keys():
                return key.cast(os.environ[var_name])
            if self._prefix_var(var_name) in os.environ.keys():
                return key.cast(os.environ[self._prefix_var(var_name)])
        return key.cast(self.get_or_default(key))

    def validate(self):
        missing = []
        for key in self.keys:
            if key.required and not self.has_value_for_key(key):
                missing.append('.'.join(key.yaml_path))
        if len(missing) > 0:
            raise MissingConfigurationError(missing)

    @property
    def db(self) -> DatabaseConfig:
        return DatabaseConfig(
            connection_string=self.get(self.keys.database.connection_string),
            connection_timeout=self.get(self.keys.database.connection_timeout),
            queue_timeout=self.get(self.keys.database.queue_timeout),
        )

    @property
    def s3(self) -> S3Config:
        return S3Config(
            url=self.get(self.keys.s3.url),
            username=self.get(self.keys.s3.username),
            password=self.get(self.keys.s3.password),
            bucket=self.get(self.keys.s3.bucket),
            region=self.get(self.keys.s3.region),
        )

    @property
    def logging(self) -> LoggingConfig:
        return LoggingConfig(
            level=self.get(self.keys.logging.level),
            global_level=self.get(self.keys.logging.global_level),
            message_format=self.get(self.keys.logging.format),
            dict_config=self.get(self.keys.logging.dict_config),
        )

    @property
    def cloud(self) -> CloudConfig:
        return CloudConfig(
            multi_tenant=self.get(self.keys.cloud.enabled),
        )

    @property
    def sentry(self) -> SentryConfig:
        return SentryConfig(
            enabled=self.get(self.keys.sentry.enabled),
            workers_dsn=self.get(self.keys.sentry.worker_dsn),
            traces_sample_rate=self.get(self.keys.sentry.traces_sample_rate),
            max_breadcrumbs=self.get(self.keys.sentry.max_breadcrumbs),
        )

    @property
    def general(self) -> GeneralConfig:
        return GeneralConfig(
            environment=self.get(self.keys.general.environment),
            client_url=self.get(self.keys.general.client_url),
            secret=self.get(self.keys.general.secret),
        )

    @property
    def aws(self) -> AWSConfig:
        return AWSConfig(
            access_key_id=self.get(self.keys.aws.access_key_id),
            secret_access_key=self.get(self.keys.aws.secret_access_key),
            region=self.get(self.keys.aws.region),
        )
